/*
 * Copyright 2012-2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.hdp;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.web.WebMvcAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.ImportResource;
import org.springframework.context.annotation.PropertySource;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;
import org.springframework.web.socket.server.standard.ServerEndpointExporter;

import redis.clients.jedis.JedisPoolConfig;

import com.hdp.customerservice.websocket.*;
import com.hdp.pi.wechat.Subscriber;

@PropertySource(value="classpath:application.${model}.properties")
@ComponentScan
@EnableAutoConfiguration
@EnableWebMvc
@EnableWebSocket
@ImportResource({"classpath*:META-INF/spring/applicationContext.xml"})
public class APP extends WebMvcAutoConfiguration
		implements WebSocketConfigurer {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(APP.class);
	
	@Value("${redis.channel}")
	private String channel;
	
	@Value("${redis.host}")
	private String host;
	
	@Value("${redis.pass}")
	private String password;
	
	@Value("${redis.port}")
	private String port;
	
	@Value("${redis.idle}")
	private String idle;
	
	@Value("${redis.maxActive}")
	private String maxActive;
	
	public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
		LOGGER.info("这里可以注册websocket路由~~~~");
		//registry.addHandler(imWebSocketEndpoint(), "/im").addInterceptors(new SimpleHandshakeInterceptor()).setAllowedOrigins("*");
	}
	
	@Bean
	public IMWebSocketEndpoint imWebSocketEndpoint() {
		return new IMWebSocketEndpoint();
	}

	@Bean
	public ServerEndpointExporter serverEndpointExporter() {
		return new ServerEndpointExporter();
	}
	
	@Bean
	RedisMessageListenerContainer container(RedisConnectionFactory connectionFactory, MessageListenerAdapter listenerAdapter){
		RedisMessageListenerContainer container = new RedisMessageListenerContainer();
		container.setConnectionFactory(connectionFactory);
		container.addMessageListener(listenerAdapter, new PatternTopic(channel));
		return container;
	}
	
	@Bean
	MessageListenerAdapter listenerAdapter(Subscriber subscriber) {
		return new MessageListenerAdapter(subscriber, "receiveMessage");
	}
	
	@Bean
	Subscriber subcriber() {
		LOGGER.info("启动监听者");
		return new Subscriber();
	}
	
	@Bean
	RedisConnectionFactory connectionFactory() {
		JedisPoolConfig poolConfig = new JedisPoolConfig();
		poolConfig.setMaxIdle(Integer.parseInt(idle));
		poolConfig.setMaxTotal(Integer.parseInt(maxActive));
		poolConfig.setTestWhileIdle(true);
		poolConfig.setNumTestsPerEvictionRun(3);
		JedisConnectionFactory connectionFactory = new JedisConnectionFactory(poolConfig);
		connectionFactory.setHostName(host);
		connectionFactory.setPassword(password);
		connectionFactory.setUsePool(true);
		connectionFactory.setTimeout(5000);
		connectionFactory.setPort(Integer.parseInt(port));
		return connectionFactory;
	}
	
	public static void main(String[] args) {
		SpringApplication.run(APP.class, args);
	}
}
